"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.storeCall = storeCall;
exports.acceptIncomingCall = acceptIncomingCall;
exports.checkCurrentCalls = checkCurrentCalls;
exports.checkWebRTCSupport = checkWebRTCSupport;
exports.declineIncomingCall = declineIncomingCall;
exports.dismissIncomingCall = dismissIncomingCall;
exports.hangupCall = hangupCall;
exports.listenForCalls = listenForCalls;
exports.placeCall = placeCall;
exports.registerClient = registerClient;
exports.storeExternalCall = storeExternalCall;
exports.UPDATE_WEBRTC_SUPPORT = exports.CHECKING_WEB_RTC_SUPPORT = exports.REMOVE_CALL = exports.UPDATE_CALL_ERROR = exports.UPDATE_CALL_STATUS = exports.STORE_CALL = exports.ANSWERED_INCOMING_CALL = exports.DISMISS_INCOMING_CALL = exports.UPDATE_STATUS = void 0;

var _assign = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/assign"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/promise"));

var _uuid = _interopRequireDefault(require("uuid"));

var _helpers = require("./helpers");

/* eslint no-use-before-define: ["error", { "functions": false }] */
var UPDATE_STATUS = 'media/UPDATE_STATUS';
exports.UPDATE_STATUS = UPDATE_STATUS;
var DISMISS_INCOMING_CALL = 'media/DISMISS_INCOMING_CALL';
exports.DISMISS_INCOMING_CALL = DISMISS_INCOMING_CALL;
var ANSWERED_INCOMING_CALL = 'media/ANSWERED_INCOMING_CALL';
exports.ANSWERED_INCOMING_CALL = ANSWERED_INCOMING_CALL;
var STORE_CALL = 'media/STORE_CALL';
exports.STORE_CALL = STORE_CALL;
var UPDATE_CALL_STATUS = 'media/UPDATE_CALL_STATUS';
exports.UPDATE_CALL_STATUS = UPDATE_CALL_STATUS;
var UPDATE_CALL_ERROR = 'media/UPDATE_CALL_ERROR';
exports.UPDATE_CALL_ERROR = UPDATE_CALL_ERROR;
var REMOVE_CALL = 'media/REMOVE_CALL';
exports.REMOVE_CALL = REMOVE_CALL;
var CHECKING_WEB_RTC_SUPPORT = 'media/CHECKING_WEB_RTC_SUPPORT';
exports.CHECKING_WEB_RTC_SUPPORT = CHECKING_WEB_RTC_SUPPORT;
var UPDATE_WEBRTC_SUPPORT = 'media/UPDATE_WEBRTC_SUPPORT';
/* Actions */

exports.UPDATE_WEBRTC_SUPPORT = UPDATE_WEBRTC_SUPPORT;

function answeredIncomingCall(callId) {
  return {
    type: ANSWERED_INCOMING_CALL,
    payload: {
      id: callId
    }
  };
}

function checkingWebRTCSupport() {
  return {
    type: CHECKING_WEB_RTC_SUPPORT
  };
}

function removeCall(id) {
  return {
    type: REMOVE_CALL,
    payload: {
      id: id
    }
  };
}

function storeCall(_ref) {
  var call = _ref.call,
      id = _ref.id,
      destination = _ref.destination;
  // check if this is a locus replaced call
  return {
    type: STORE_CALL,
    payload: {
      id: id,
      call: (0, _helpers.constructCallObject)(call),
      destination: destination
    }
  };
}

function updateCallError(_ref2) {
  var call = _ref2.call,
      error = _ref2.error,
      id = _ref2.id;
  return {
    type: UPDATE_CALL_ERROR,
    payload: {
      id: id || call.id,
      call: (0, _helpers.constructCallObject)(call),
      error: error
    }
  };
}

function updateCallStatus(_ref3) {
  var call = _ref3.call,
      id = _ref3.id,
      eventName = _ref3.eventName;
  return {
    type: UPDATE_CALL_STATUS,
    payload: {
      id: id,
      call: (0, _helpers.constructCallObject)(call),
      eventName: eventName
    }
  };
}

function updateStatus(status) {
  return {
    type: UPDATE_STATUS,
    payload: {
      status: status
    }
  };
}

function updateWebRTCSupport(supported) {
  return {
    type: UPDATE_WEBRTC_SUPPORT,
    payload: {
      supported: supported
    }
  };
}
/* Exported Actions */

/**
 * Accepts an incoming call
 *
 * @export
 * @param {object} incomingCall
 * @param {object} constraints rtc constraints for getmedia request
 * @param {boolean} constraints.audio
 * @param {object} constraints.video
 * @returns {Promise}
 */


function acceptIncomingCall(incomingCall, constraints) {
  return function (dispatch) {
    return incomingCall.instance.answer({
      constraints: constraints,
      offerOptions: {
        offerToReceiveVideo: true,
        offerToReceiveAudio: true
      }
    }).then(function () {
      dispatch(updateCallStatus({
        call: incomingCall.instance,
        id: incomingCall.id
      }));
      return incomingCall;
    });
  };
}
/**
 * Does a spark API request for existing calls and stores them
 * @param {object} sparkInstance
 * @returns {Promise}
 */


function checkCurrentCalls(sparkInstance) {
  return function (dispatch) {
    return sparkInstance.phone.listActiveCalls().then(function (results) {
      return _promise.default.all(results.map(function (callObject) {
        return _promise.default.resolve(processCall({
          dispatch: dispatch,
          call: callObject
        }));
      }));
    });
  };
}
/**
 * Check for browser webRTC support
 *
 *  @param {object} sparkInstance
 *  @returns {Promise}
 */


function checkWebRTCSupport(sparkInstance) {
  return function (dispatch) {
    if (sparkInstance.phone) {
      dispatch(checkingWebRTCSupport());
      return sparkInstance.phone.isCallingSupported().then(function (supported) {
        return dispatch(updateWebRTCSupport(supported));
      });
    }

    return _promise.default.resolve();
  };
}
/**
 * Declines an incoming call
 *
 * @export
 * @param {object} incomingCall
 * @returns {Thunk}
 */


function declineIncomingCall(incomingCall) {
  return function (dispatch) {
    incomingCall.instance.reject();
    dispatch(removeCall(incomingCall.id));
    return _promise.default.resolve();
  };
}
/**
 * Dismisses an incoming call, marking it so.
 * Note: This does not decline the call, just for tracking purposes
 * @param {string} id
 * @returns {object}
 */


function dismissIncomingCall(id) {
  return {
    type: DISMISS_INCOMING_CALL,
    payload: {
      id: id
    }
  };
}
/**
 * Hangs up and removes call
 * @param {object} call
 * @returns {Promise}
 */


function hangupCall(_ref4) {
  var call = _ref4.call,
      id = _ref4.id;
  return function (dispatch) {
    // Don't update call states after hangup
    call.off(); // Only call.hangup() when local user wants to hangup.
    // SDK handles .hangup() when call becomes `inactive`

    return call.hangup().then(function () {
      return dispatch(removeCall(id));
    }).catch(function () {
      return dispatch(removeCall(id));
    });
  };
}
/**
 * Listens for all call events from phone plugin and handles them
 * @param {Object} sparkInstance
 * @returns {Promise}
 */


function listenForCalls(sparkInstance) {
  return function (dispatch) {
    dispatch(updateStatus({
      isListening: true,
      isListeningToIncoming: true
    }));
    return _promise.default.all([handleIncomingCalls(sparkInstance), handleCreatedCalls(sparkInstance, dispatch)]);
  };
}
/**
 * Call a user with an email address or userId
 *
 * @export
 * @param {Object} sparkInstance
 * @param {String} data.destination
 * @param {Object} data.options
 * @returns {Promise}
 */


function placeCall(sparkInstance, _ref5) {
  var destination = _ref5.destination,
      _ref5$options = _ref5.options,
      options = _ref5$options === void 0 ? {
    audio: true,
    video: true
  } : _ref5$options;
  return function (dispatch) {
    var call, id;

    try {
      call = sparkInstance.phone.dial(destination, (0, _assign.default)(options, {
        offerOptions: {
          offerToReceiveVideo: true,
          offerToReceiveAudio: true
        }
      }));

      var _processCall = processCall({
        dispatch: dispatch,
        call: call
      });

      call = _processCall.call;
      id = _processCall.id;
    } catch (e) {// Ignore, will be caught on call.on('error')
    }

    return _promise.default.resolve({
      call: call,
      id: id
    });
  };
}
/**
 * Registers the client for media/phone
 * @param {object} sparkInstance
 * @returns {Promise}
 */


function registerClient(sparkInstance) {
  return function (dispatch, getState) {
    var _getState = getState(),
        media = _getState.media;

    if (!media.status.isRegistered && !media.status.isRegistering) {
      dispatch(updateStatus({
        isRegistering: true
      }));
      return sparkInstance.phone.register().then(function () {
        var status = {
          isRegistered: true,
          isRegistering: false
        };
        dispatch(updateStatus(status));
        return status;
      });
    }

    return _promise.default.resolve(media.status);
  };
}
/**
 * Process and store a call object from outside widgets
 * @param {Object} call
 * @returns {Function}
 */


function storeExternalCall(call) {
  return function (dispatch) {
    var _processCall2 = processCall({
      dispatch: dispatch,
      call: call
    }),
        id = _processCall2.id;

    return {
      id: id,
      call: call
    };
  };
}
/* Helper Functions */

/**
 * Binds call events and dispatches to store
 * @param {*} params.dispatch
 * @param {Object} params.call
 * @param {String} params.id
 * @returns {Object} {call, id}
 */


function bindEvents(_ref6) {
  var dispatch = _ref6.dispatch,
      call = _ref6.call,
      id = _ref6.id;

  var callId = id || _uuid.default.v4();

  var updateCall = function updateCall(callInstance, eventName) {
    return dispatch(updateCallStatus({
      call: callInstance,
      id: callId,
      eventName: eventName
    }));
  };

  call.once('active', function () {
    return dispatch(updateCall(call, 'active'));
  });
  call.once('inactive', function () {
    return dispatch(removeCall(callId));
  });
  ['change:state', 'change:sendingAudio', 'change:receivingAudio', 'change:sendingVideo', 'change:receivingVideo', 'change:activeParticipantsCount', 'remoteAudioMuted:change', 'remoteVideoMuted:change', 'remoteMediaStream:change', 'localMediaStream:change', 'change:joinedOnThisDevice', 'membership:notified', 'membership:disconnected', 'membership:change', 'memberships:add', 'memberships:remove'].map(function (event) {
    return call.on(event, function () {
      return updateCall(call, event);
    });
  });

  if (call.direction === 'out') {
    // If we rejected the call elsewhere, remove it
    call.on('membership:declined', function () {
      if (call.me.state === 'rejected') {
        dispatch(dismissIncomingCall(callId));
      }
    });
    call.on('membership:change', function () {
      // Once we are connected, our connected call state will handle disconnects
      if (call.me.state === 'connected') {
        dispatch(answeredIncomingCall(callId)); // incomingCall.joinedOnThisDevice takes a few ticks so we have to
        // calculate it ourselves for now

        if (call.locus.self.deviceUrl !== call.spark.internal.device.url) {
          // We are not connected on this device, dismiss
          dispatch(dismissIncomingCall(callId));
        }
      }
    });
  }

  call.on('membership:declined', function () {
    if (call.isCall && call.remoteMember && call.remoteMember.state === 'declined') {
      // Hang up if the remote person declines the call
      return dispatch(hangupCall({
        call: call,
        id: callId
      }));
    }

    if (call.me.state === 'rejected') {
      dispatch(dismissIncomingCall(callId));
    }

    return updateCall(call, 'membership:declined');
  });
  call.on('error', function (error) {
    return dispatch(updateCallError({
      call: call,
      error: error,
      id: call.id || callId
    }));
  });
  return {
    call: call,
    id: callId
  };
}
/**
 * Processes a call by binding events and storing
 * @param {*} params.dispatch
 * @param {Object} params.call
 * @returns {Object} {id, call}
 */


function processCall(_ref7) {
  var dispatch = _ref7.dispatch,
      call = _ref7.call;

  var _bindEvents = bindEvents({
    dispatch: dispatch,
    call: call
  }),
      id = _bindEvents.id;

  dispatch(storeCall({
    call: call,
    id: id
  }));
  return {
    id: id,
    call: call
  };
}
/**
 * Sets up event listener for the 'call:incoming' phone event
 * @param {object} sparkInstance
 * @param {object} dispatch
 * @returns {Promise}
 */


function handleIncomingCalls(sparkInstance) {
  return sparkInstance.phone.on('call:incoming', function (incomingCall) {
    // If we make an outbound call via the widget, when the remote party answers,
    // we get a call:incoming notification when they connect
    if (incomingCall.direction === 'out') {
      return false;
    }

    return incomingCall.acknowledge();
  });
}
/**
 * Sets up event listener for the 'call:created' phone event
 * @param {object} sparkInstance
 * @param {object} dispatch
 * @returns {Promise}
 */


function handleCreatedCalls(sparkInstance, dispatch) {
  return sparkInstance.phone.on('call:created', function (createdCall) {
    // Do not store call when a created inactive call comes in
    if (createdCall.state !== 'inactive') {
      processCall({
        dispatch: dispatch,
        call: createdCall
      });
    }

    return _promise.default.resolve(createdCall);
  });
}