"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty2 = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty2(exports, "__esModule", {
  value: true
});

exports.releaseCamera = releaseCamera;
exports.constructCallObject = constructCallObject;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/define-property"));

var _defineProperties = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/define-properties"));

var _getOwnPropertyDescriptors = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-descriptors"));

var _getOwnPropertyDescriptor = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-descriptor"));

var _getOwnPropertySymbols = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-symbols"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _defineProperty3 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/defineProperty"));

var _now = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/date/now"));

var _immutable = require("immutable");

function ownKeys(object, enumerableOnly) { var keys = (0, _keys.default)(object); if (_getOwnPropertySymbols.default) { var symbols = (0, _getOwnPropertySymbols.default)(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return (0, _getOwnPropertyDescriptor.default)(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty3.default)(target, key, source[key]); }); } else if (_getOwnPropertyDescriptors.default) { (0, _defineProperties.default)(target, (0, _getOwnPropertyDescriptors.default)(source)); } else { ownKeys(Object(source)).forEach(function (key) { (0, _defineProperty2.default)(target, key, (0, _getOwnPropertyDescriptor.default)(source, key)); }); } } return target; }

var Membership = (0, _immutable.Record)({
  audioMuted: false,
  isInitiator: true,
  hydraId: '',
  id: '',
  state: '',
  videoMuted: false
});
/**
 * Constructs a call state with properties the widget uses
 *
 * @export
 * @param {any} call
 * @param {Error} rawError
 * @returns {object}
 */

function constructCallStatus(call) {
  var isConnected, isIncoming, isInitiated, isRinging;
  var remoteAudioMuted = call.remoteAudioMuted,
      remoteVideoMuted = call.remoteVideoMuted;

  try {
    // Ringing means we are calling out to 1:1 and the party hasn't answered
    isRinging = call.remoteMember && call.remoteMember.state === 'notified'; // Connected means both parties are active on a 1:1 or joined in group
    // isCall is true when the call is 1:1

    if (typeof call.isCall === 'boolean') {
      if (call.isCall) {
        isInitiated = !!call.locus;
        isConnected = !!call.remoteMember && call.remoteMember.state === 'connected' && !!call.joinedOnThisDevice;
      } else {
        // Group calls are immediately initiated
        isInitiated = true;
        isConnected = !!call.joinedOnThisDevice;
      }
    } else {
      isInitiated = false;
      isConnected = false;
    }

    isIncoming = call.direction === 'in' && !call.hasJoinedOnThisDevice;
  } catch (e) {// Do nothing
  }

  return {
    isCall: call.isCall,
    isActive: call.isActive,
    hasJoinedOnThisDevice: call.joinedOnThisDevice,
    isReceivingAudio: call.receivingAudio,
    isReceivingVideo: call.receivingVideo,
    isSendingAudio: call.sendingAudio,
    isSendingVideo: call.sendingVideo,
    isDeclined: call.me && call.me.state === 'declined',
    isInitiated: isInitiated,
    isConnected: isConnected,
    isRinging: isRinging,
    isIncoming: isIncoming,
    remoteAudioMuted: remoteAudioMuted,
    remoteVideoMuted: remoteVideoMuted
  };
}

function releaseCamera(mediaStream) {
  return mediaStream.getTracks().forEach(function (track) {
    return track.stop();
  });
}

function constructCallObject(call) {
  var remoteAudioStream = null;
  var remoteVideoStream = null;
  var activeParticipantsCount = 0;
  var startTime;
  var memberships = [];

  try {
    if (call && call.remoteMediaStream && call.remoteMediaStream.getAudioTracks) {
      remoteAudioStream = new MediaStream(call.remoteMediaStream.getAudioTracks());
    }

    if (call && call.remoteMediaStream && call.remoteMediaStream.getVideoTracks) {
      remoteVideoStream = new MediaStream(call.remoteMediaStream.getVideoTracks());
    }

    call.memberships.forEach(function (m) {
      // only count user as active participant if they are connected
      if (m.state === 'connected') {
        activeParticipantsCount += 1;
      }

      memberships.push(Membership({
        audioMuted: m.audioMuted,
        isInitiator: m.isInitiator,
        hydraId: m.personId,
        id: m.personUuid,
        state: m.state,
        videoMuted: m.videoMuted
      }));
    });

    if (call.locus && call.locus.fullState && call.locus.fullState.lastActive) {
      startTime = Date.parse(call.locus.fullState.lastActive);
    } else {
      startTime = (0, _now.default)();
    }
  } catch (e) {// Do nothing
  }

  return _objectSpread({
    instance: call,
    locusUrl: call.locusUrl,
    direction: call.direction,
    remoteMediaStream: call.remoteMediaStream,
    localMediaStream: call.localMediaStream,
    startTime: startTime,
    memberships: memberships,
    activeParticipantsCount: activeParticipantsCount,
    remoteAudioStream: remoteAudioStream,
    remoteVideoStream: remoteVideoStream
  }, constructCallStatus(call));
}