"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = reducer;
exports.initialState = exports.CallRecord = void 0;

var _immutable = require("immutable");

var _actions = require("./actions");

var ErrorObject = (0, _immutable.Record)({
  name: '',
  message: ''
});
var CallRecord = (0, _immutable.Record)({
  instance: null,
  activeParticipantsCount: 0,
  direction: '',
  startTime: null,
  remoteAudioStream: null,
  remoteVideoStream: null,
  remoteMediaStream: null,
  localMediaStream: null,
  error: null,
  memberships: new _immutable.List(),
  locusUrl: '',
  id: '',
  isCall: false,
  isActive: false,
  isIncoming: false,
  isInitiated: false,
  isConnected: false,
  isAnswered: false,
  isDismissed: false,
  isDeclined: false,
  isRinging: false,
  isReceivingAudio: false,
  isReceivingVideo: false,
  isSendingAudio: false,
  isSendingVideo: false,
  remoteAudioMuted: false,
  remoteVideoMuted: false,
  hasJoinedOnThisDevice: false,
  hasError: false
});
exports.CallRecord = CallRecord;
var WebRTCStatus = (0, _immutable.Record)({
  hasCheckedSupport: false,
  isSupported: null
});
var MediaStatus = (0, _immutable.Record)({
  isListening: false,
  isListeningToIncoming: false,
  isRegistered: false,
  isRegistering: false
});
var InitialState = (0, _immutable.Record)({
  byDestination: (0, _immutable.Map)(),
  byLocusUrl: (0, _immutable.Map)(),
  byId: (0, _immutable.Map)(),
  webRTC: WebRTCStatus(),
  status: MediaStatus()
});
var initialState = new InitialState();
exports.initialState = initialState;

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments.length > 1 ? arguments[1] : undefined;

  switch (action.type) {
    case _actions.UPDATE_STATUS:
      return state.mergeIn(['status'], action.payload.status);

    case _actions.STORE_CALL:
      {
        var _action$payload = action.payload,
            call = _action$payload.call,
            destination = _action$payload.destination,
            id = _action$payload.id;

        if (id) {
          var locusUrl = call.locusUrl;
          var updatedState = state.setIn(['byId', id], CallRecord(call).set('id', id));

          if (destination) {
            updatedState = updatedState.setIn(['byDestination', destination], id);
          }

          if (locusUrl) {
            updatedState = updatedState.setIn(['byLocusUrl', locusUrl], id);
          }

          return updatedState;
        }

        return state;
      }

    case _actions.UPDATE_CALL_STATUS:
      {
        var _updatedState = state;
        var _action$payload2 = action.payload,
            _call = _action$payload2.call,
            _id = _action$payload2.id;

        if (_id) {
          _updatedState = _updatedState.mergeIn(['byId', _id], _call);

          if (_call.instance) {
            // Need to set instance object so it isn't manipulated by immutable
            _updatedState = _updatedState.setIn(['byId', _id, 'instance'], _call.instance);
          }
        } // Handle locus replacement


        var instance = _call.instance;

        if (instance && instance.locus && instance.locus.url && state.getIn(['byLocusUrl', instance.locus.url]) !== _id) {
          // remove old locusUrl if it exists
          var oldUrl = state.get('byLocusUrl').keyOf(_id);

          if (oldUrl) {
            _updatedState = _updatedState.removeIn(['byLocusUrl', oldUrl]);
          } // add new URL


          _updatedState = _updatedState.setIn(['byLocusUrl', instance.locus.url], _id);
        }

        return _updatedState;
      }

    case _actions.UPDATE_CALL_ERROR:
      {
        var _action$payload3 = action.payload,
            _call2 = _action$payload3.call,
            _id2 = _action$payload3.id,
            error = _action$payload3.error;

        if (_id2) {
          return state.mergeIn(['byId', _id2], _call2).setIn(['byId', _id2, 'error'], new ErrorObject(error));
        }

        return state;
      }

    case _actions.DISMISS_INCOMING_CALL:
      return state.setIn(['byId', action.payload.id, 'isDismissed'], true);

    case _actions.REMOVE_CALL:
      {
        var _updatedState2 = state;
        var byDestinationKey = state.get('byDestination').keyOf(action.payload.id);
        var byLocusUrl = state.get('byLocusUrl').keyOf(action.payload.id); // Also need to check if locusUrl exists for any other call objects

        state.get('byId').forEach(function (c, id) {
          if (c.locusUrl === byLocusUrl) {
            _updatedState2 = _updatedState2.deleteIn(['byId', id]).deleteIn(['byDestination', state.get('byDestination').keyOf(id)]).deleteIn(['byId', id]);
          }
        });
        _updatedState2 = _updatedState2.deleteIn(['byId', action.payload.id]).deleteIn(['byDestination', byDestinationKey]).deleteIn(['byLocusUrl', byLocusUrl]);
        return _updatedState2;
      }

    case _actions.ANSWERED_INCOMING_CALL:
      return state.setIn(['byId', action.payload.id, 'isAnswered'], true);

    case _actions.CHECKING_WEB_RTC_SUPPORT:
      return state.setIn(['webRTC', 'hasCheckedSupport'], true);

    case _actions.UPDATE_WEBRTC_SUPPORT:
      return state.setIn(['webRTC', 'isSupported'], action.payload.supported);

    default:
      return state;
  }
}